library angular2.src.core.compiler.shadow_dom_strategy_dart;

import 'package:angular2/src/facade/lang.dart'
    show Type, isBlank, isPresent, int;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, MapWrapper, Map;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import './view.dart' show View;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_emulation/light_dom.dart' show LightDom;
import './shadow_dom_emulation/shadow_css.dart' show ShadowCss;
import './style_inliner.dart' show StyleInliner;
import './style_url_resolver.dart' show StyleUrlResolver;

class ShadowDomStrategy {
  attachTemplate(el, View view) {}
  constructLightDom(View lightDomView, View shadowDomView, el) {}
  List<Type> polyfillDirectives() {
    return null;
  }
  transformStyleText(String cssText, String baseUrl, Type component) {}
  handleStyleElement(styleEl) {}
  shimContentElement(Type component, element) {}
  shimHostElement(Type component, element) {}
}
class EmulatedUnscopedShadowDomStrategy extends ShadowDomStrategy {
  StyleUrlResolver _styleUrlResolver;
  var _lastInsertedStyle;
  var _styleHost;
  EmulatedUnscopedShadowDomStrategy(
      StyleUrlResolver styleUrlResolver, styleHost)
      : super() {
    this._styleUrlResolver = styleUrlResolver;
    this._styleHost = styleHost;
  }
  attachTemplate(el, View view) {
    DOM.clearNodes(el);
    _moveViewNodesIntoParent(el, view);
  }
  constructLightDom(View lightDomView, View shadowDomView, el) {
    return new LightDom(lightDomView, shadowDomView, el);
  }
  List<Type> polyfillDirectives() {
    return [Content];
  }
  transformStyleText(String cssText, String baseUrl, Type component) {
    return this._styleUrlResolver.resolveUrls(cssText, baseUrl);
  }
  handleStyleElement(styleEl) {
    DOM.remove(styleEl);
    var cssText = DOM.getText(styleEl);
    if (!MapWrapper.contains(_sharedStyleTexts, cssText)) {
      MapWrapper.set(_sharedStyleTexts, cssText, true);
      this._insertStyleElement(this._styleHost, styleEl);
    }
  }
  _insertStyleElement(host, style) {
    if (isBlank(this._lastInsertedStyle)) {
      var firstChild = DOM.firstChild(host);
      if (isPresent(firstChild)) {
        DOM.insertBefore(firstChild, style);
      } else {
        DOM.appendChild(host, style);
      }
    } else {
      DOM.insertAfter(this._lastInsertedStyle, style);
    }
    this._lastInsertedStyle = style;
  }
}
class EmulatedScopedShadowDomStrategy
    extends EmulatedUnscopedShadowDomStrategy {
  StyleInliner _styleInliner;
  EmulatedScopedShadowDomStrategy(
      StyleInliner styleInliner, StyleUrlResolver styleUrlResolver, styleHost)
      : super(styleUrlResolver, styleHost) {
    this._styleInliner = styleInliner;
  }
  transformStyleText(String cssText, String baseUrl, Type component) {
    cssText = this._styleUrlResolver.resolveUrls(cssText, baseUrl);
    var css = this._styleInliner.inlineImports(cssText, baseUrl);
    if (PromiseWrapper.isPromise(css)) {
      return css.then((css) {
        return _shimCssForComponent(css, component);
      });
    } else {
      return _shimCssForComponent(css, component);
    }
  }
  handleStyleElement(styleEl) {
    DOM.remove(styleEl);
    this._insertStyleElement(this._styleHost, styleEl);
  }
  shimContentElement(Type component, element) {
    var id = _getComponentId(component);
    var attrName = _getContentAttribute(id);
    DOM.setAttribute(element, attrName, '');
  }
  shimHostElement(Type component, element) {
    var id = _getComponentId(component);
    var attrName = _getHostAttribute(id);
    DOM.setAttribute(element, attrName, '');
  }
}
class NativeShadowDomStrategy extends ShadowDomStrategy {
  StyleUrlResolver _styleUrlResolver;
  NativeShadowDomStrategy(StyleUrlResolver styleUrlResolver) : super() {
    this._styleUrlResolver = styleUrlResolver;
  }
  attachTemplate(el, View view) {
    _moveViewNodesIntoParent(DOM.createShadowRoot(el), view);
  }
  constructLightDom(View lightDomView, View shadowDomView, el) {
    return null;
  }
  List<Type> polyfillDirectives() {
    return [];
  }
  transformStyleText(String cssText, String baseUrl, Type component) {
    return this._styleUrlResolver.resolveUrls(cssText, baseUrl);
  }
}
_moveViewNodesIntoParent(parent, view) {
  for (var i = 0; i < view.nodes.length; ++i) {
    DOM.appendChild(parent, view.nodes[i]);
  }
}
Map<Type, int> _componentUIDs = MapWrapper.create();
int _nextComponentUID = 0;
Map<String, bool> _sharedStyleTexts = MapWrapper.create();
_getComponentId(Type component) {
  var id = MapWrapper.get(_componentUIDs, component);
  if (isBlank(id)) {
    id = _nextComponentUID++;
    MapWrapper.set(_componentUIDs, component, id);
  }
  return id;
}
_getHostAttribute(int id) {
  return '''_nghost-${id}''';
}
_getContentAttribute(int id) {
  return '''_ngcontent-${id}''';
}
String _shimCssForComponent(String cssText, Type component) {
  var id = _getComponentId(component);
  var shadowCss = new ShadowCss();
  return shadowCss.shimCssText(
      cssText, _getContentAttribute(id), _getHostAttribute(id));
}
resetShadowDomCache() {
  MapWrapper.clear(_componentUIDs);
  _nextComponentUID = 0;
  MapWrapper.clear(_sharedStyleTexts);
}
