library angular2.src.forms.model_dart;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show Map, StringMapWrapper;
import './validators.dart' show nullValidator, controlGroupValidator;

const VALID = "VALID";
const INVALID = "INVALID";
class AbstractControl {
  dynamic _value;
  String _status;
  var _errors;
  bool _dirty;
  ControlGroup _parent;
  Function validator;
  AbstractControl([Function validator = nullValidator]) {
    this.validator = validator;
    this._dirty = true;
  }
  bool get active {
    return true;
  }
  get value {
    this._updateIfNeeded();
    return this._value;
  }
  get status {
    this._updateIfNeeded();
    return this._status;
  }
  get valid {
    this._updateIfNeeded();
    return identical(this._status, VALID);
  }
  get errors {
    this._updateIfNeeded();
    return this._errors;
  }
  setParent(parent) {
    this._parent = parent;
  }
  _updateIfNeeded() {}
  _updateParent() {
    if (isPresent(this._parent)) {
      this._parent._controlChanged();
    }
  }
}
class Control extends AbstractControl {
  Control(dynamic value, [Function validator = nullValidator])
      : super(validator) {
    this._value = value;
  }
  updateValue(dynamic value) {
    this._value = value;
    this._dirty = true;
    this._updateParent();
  }
  _updateIfNeeded() {
    if (this._dirty) {
      this._dirty = false;
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
}
class ControlGroup extends AbstractControl {
  var controls;
  ControlGroup(controls, [Function validator = controlGroupValidator])
      : super(validator) {
    this.controls = controls;
    this._setParentForControls();
  }
  _setParentForControls() {
    StringMapWrapper.forEach(this.controls, (control, name) {
      control.setParent(this);
    });
  }
  _updateIfNeeded() {
    if (this._dirty) {
      this._dirty = false;
      this._value = this._reduceValue();
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
  _reduceValue() {
    var newValue = {};
    StringMapWrapper.forEach(this.controls, (control, name) {
      if (control.active) {
        newValue[name] = control.value;
      }
    });
    return newValue;
  }
  _controlChanged() {
    this._dirty = true;
    this._updateParent();
  }
}
class OptionalControl {
  Control _control;
  bool _cond;
  OptionalControl(Control control, bool cond) : super() {
    this._control = control;
    this._cond = cond;
  }
  bool get active {
    return this._cond;
  }
  get value {
    return this._control.value;
  }
  get status {
    return this._control.status;
  }
  get errors {
    return this._control.errors;
  }
  set validator(v) {
    this._control.validator = v;
  }
  get validator {
    return this._control.validator;
  }
  set cond(bool value) {
    this._cond = value;
    this._control._updateParent();
  }
  bool get cond {
    return this._cond;
  }
  updateValue(dynamic value) {
    this._control.updateValue(value);
  }
  setParent(parent) {
    this._control.setParent(parent);
  }
}
