library angular2.test.core.compiler.pipeline.shim_shadow_dom_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, expect, it, iit, ddescribe, el;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/shim_shadow_dom.dart'
    show ShimShadowDom;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/facade/lang.dart' show Type, isBlank, stringify;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;

main() {
  describe('ShimShadowDom', () {
    createPipeline(bool ignoreBindings) {
      var annotation = new Component(selector: 'selector');
      var meta = new DirectiveMetadata(SomeComponent, annotation);
      var shimShadowDom = new ShimShadowDom(meta, new FakeStrategy());
      return new CompilePipeline([
        new MockStep((parent, current, control) {
          current.ignoreBindings = ignoreBindings;
        }),
        new MockStep((parent, current, control) {
          var el = current.element;
          if (DOM.hasClass(el, 'host')) {
            current.componentDirective =
                new DirectiveMetadata(SomeComponent, null);
          }
        }),
        shimShadowDom
      ]);
    }
    it('should add the content attribute to content element', () {
      var pipeline = createPipeline(false);
      var results = pipeline.process(el('<div></div>'));
      expect(DOM.getAttribute(results[0].element, 'SomeComponent-content'))
          .toEqual('');
      expect(isBlank(
              DOM.getAttribute(results[0].element, 'SomeComponent-host')))
          .toBeTruthy();
    });
    it('should add both the content and host attributes to host element', () {
      var pipeline = createPipeline(false);
      var results = pipeline.process(el('<div class="host"></div>'));
      expect(DOM.getAttribute(results[0].element, 'SomeComponent-content'))
          .toEqual('');
      expect(DOM.getAttribute(results[0].element, 'SomeComponent-host'))
          .toEqual('');
    });
    it('should do nothing when ignoreBindings is true', () {
      var pipeline = createPipeline(true);
      var results = pipeline.process(el('<div class="host"></div>'));
      expect(isBlank(
              DOM.getAttribute(results[0].element, 'SomeComponent-content')))
          .toBeTruthy();
      expect(isBlank(
              DOM.getAttribute(results[0].element, 'SomeComponent-host')))
          .toBeTruthy();
    });
  });
}
class FakeStrategy extends ShadowDomStrategy {
  FakeStrategy() : super() {}
  shimContentElement(Type component, element) {
    var attrName = stringify(component) + '-content';
    DOM.setAttribute(element, attrName, '');
  }
  shimHostElement(Type component, element) {
    var attrName = stringify(component) + '-host';
    DOM.setAttribute(element, attrName, '');
  }
}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
class SomeComponent {}
